/******************************************/
/***************** ABOUT ******************/
/*****************************************/
/*20 Mar 2020
 *This program attempts to control the supply to utilities in the guest bathroom in such a way that 
 * 
 * -the exhaust fan come on after 3 minutes of a person entering the bathroom
 * -the solar light remains off when there is no presence in the bathroom (to conserve solar generated electricity)
 * -once the solar light comes on it stays on for 5 minutes before it goes off unless it senses presence.
 * -Ensure bathroom light does not come on even if switch is ON during the mornig hours (i.e 10 AM - 5 PM)
 * 
 * 26 Mar 2020 Ver 1.0 completed and archived
 */
 
/******************************************/
/***************** CREDITS ****************/
/*****************************************/

/* https://www.arduino.cc/en/tutorial/smoothing for the averaging code

/******************************************/
/**************PIN CONNECTIONS ************/
/*****************************************/
/*****************************************
CLOCK Module 
GND to Arduino Ground
Vcc to Arduino +5V
SCL to Arduino A5       
SDA to Arduino A4       

LCD Module
GND to Arduino Ground
Vcc to Arduino +5V
SCK to Arduino D11     
CLK to Arduino D12     
D   to Arduino D10     

RELAYS
4 relays connected from D2 to D5

HC06 Module
Vcc to Ardunio 3.3V
GND to Arduino Ground
Tx  to Arduino D0 (Rx)
Rx  to Arduino D1 (Tx)

DISTANCE Sensor
When HC 06 is not being used use pins D0 and D1 as shown:-
Echo - D8
Trig - D7
****************************************/

#include "Wire.h"                 // used with HC-05
//#include <VirtualWire.h>        // Line introduced for RF transmitter section
#define DS3231_I2C_ADDRESS 0x68   // for clock                                      when using arduino uno
//#include <IRremote.h>           // for IR diodie
#include <SoftwareSerial.h>       // for HC-06
#include <LiquidCrystal_SR.h>     // include the library

//IRsend irsend;
// Defining LCD and Pins for interfacing. 
   
LiquidCrystal_SR lcd(10, 12, 11); // Pin 10 - Data Enable/ SER, Pin 12 - Clock/SCL, Pin 11 -SCK

//SoftwareSerial mySerial(0, 1);    // RX, TX for HC-05 module
//Pin ENABLE of HC-05 is only used by connecting it to pin 12 of arduino when it has to be configured
//char val; // variable to receive data from the serial port
char buf[16];
//String voice;                                                                  // a string to hold incoming data
int clearflag = 0;  
int timer_count = 0;
int elapsedtimecount = 0;
int elapsedtimelimit = 200;                                                    // sets the duration of elapsedtime counter
//int charcount;

const int numReadings = 10;     // Number of samples to average
int readings[numReadings];      // the readings from the analog input
int readIndex = 0;              // the index of the current reading
int total = 0;                  // the running total
int average = 0;                // the average

/*-----( Declare Constants )-----*/
#define RELAY_ON 0
#define RELAY_OFF 1

/*-----( Declare objects )-----*/
//int RECV_PIN = 11;
int FlagTube = 0;                                               // Used to monitor state of the bathroom tube lights
int FlagSolar = 0;                                              // Used to set state of solar lights
int FlagPresence = 0;                                           // Used to indicate presence of person

//int Flagpin_1 =0; // FLAGPINS ARE USED PRIMARILY A FLAGS TO DECIDE IF A RELAY IS TO BE SWITCHED ON OR OFF DEPENDING ON THEIR LAST STATE
//int Flagpin_2 =0;
//int Flagpin_3 =0;
//int Flagpin_4 =0;
//int Flagpin_5 =0;
//int Flaglights =0; 
//int Flagfans =0;

/*-----( Declare Variables )-----*/
#define Relay_1  5                                                // Bathroom Tube light
#define Relay_2  4                                                // Arduino Digital I/O pin number           **** DEFECTIVE ****
#define Relay_3  2                                                // Bathroom Exhaust fan
#define Relay_4  3                                                // ** bathroom solar light **
//#define Relay_5  4 
//#define Relay_6  5
//#define Relay_7  6                                               


/*-----( Declare Variables )-----*/
#define trigPin 7                                                 // For use of distance sensor
#define echoPin 8

// whether the string is complete         
boolean stringComplete = false; 


// Convert normal decimal numbers to binary coded decimal
byte decToBcd(byte val)
{
  return( (val/10*16) + (val%10) );
}
// Convert binary coded decimal to normal decimal numbers
byte bcdToDec(byte val)
{
  return( (val/16*10) + (val%16) );
}


void setup() 
{
//-------( Initialize Pins so relays are inactive at reset)----
  digitalWrite(Relay_1, RELAY_OFF);
  digitalWrite(Relay_2, RELAY_OFF);
  digitalWrite(Relay_3, RELAY_OFF);
  digitalWrite(Relay_4, RELAY_OFF); 
  //digitalWrite(Relay_5, RELAY_OFF);
  
    
//---( THEN set pins as outputs )----  
  pinMode(Relay_1, OUTPUT);   
  pinMode(Relay_2, OUTPUT);  
  pinMode(Relay_3, OUTPUT);  
  pinMode(Relay_4, OUTPUT);  
//  pinMode(Relay_5, OUTPUT); 
//  pinMode(Relay_6, OUTPUT);
//  pinMode(Relay_7, OUTPUT);
 

 //
  //pinMode(ledpin, OUTPUT);  // Onboard LED as OUTPUT
  Wire.begin();               // for clock display
  Serial.begin(9600);         // start serial communication at 9600bps

  //lcd.begin(16,2);           // 16 characters, 2 rows
  lcd.begin(20,4);             // 20 characters, 4 rows
  lcd.cursor();
  lcd.clear();

 // set the initial time here:
 // DS3231 seconds, minutes, hours, day of week, date, month, year
 //setDS3231time(30,31,20,6,16,9,16);

  pinMode(trigPin, OUTPUT);                                                       // for distance sensor
  pinMode(echoPin, INPUT);                                                        // for distance sensor
  //mySerial.begin(9600);
  Serial.begin(9600);

  lcd.setCursor(4,0);
  lcd.print("SMART SWITCH");
  lcd.setCursor(2,2);
  lcd.print("R RAMESH");
  lcd.setCursor(6,3);
  lcd.print("(C)2020");
  delay (8000);
  lcd.clear();
}                                                                                     // END OF SETUP


//void alllightson()
//     {
//     digitalWrite(Relay_1, RELAY_ON); 
//     
//     delay(1000);
//     digitalWrite(Relay_2, RELAY_ON);
//     Flagpin_2 =1; 
//     delay (1000);
//     }
//     
//void alllightsoff()
//    {
//     digitalWrite(Relay_1, RELAY_OFF); 
//     Flagpin_1 =0;
//     delay(1000);
//     digitalWrite(Relay_2, RELAY_OFF);
//     Flagpin_2 =0; 
//     delay (1000);
//     }


/******************************************/
/******** clock related function **********/
/*****************************************/

//
void setDS3231time(byte second, byte minute, byte hour, byte dayOfWeek, byte dayOfMonth, byte month, byte year)
{
  // sets time and date data to DS3231
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0); // set next input to start at the seconds register
  Wire.write(decToBcd(second)); // set seconds
  Wire.write(decToBcd(minute)); // set minutes
  Wire.write(decToBcd(hour)); // set hours
  Wire.write(decToBcd(dayOfWeek)); // set day of week (1=Sunday, 7=Saturday)
  Wire.write(decToBcd(dayOfMonth)); // set date (1 to 31)
  Wire.write(decToBcd(month)); // set month
  Wire.write(decToBcd(year)); // set year (0 to 99)
  Wire.endTransmission();
}

void readDS3231time(byte *second,byte *minute,byte *hour,byte *dayOfWeek,byte *dayOfMonth,byte *month,byte *year)
{
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0); // set DS3231 register pointer to 00h
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 7);
  // request seven bytes of data from DS3231 starting from register 00h
  *second = bcdToDec(Wire.read() & 0x7f);
  *minute = bcdToDec(Wire.read());
  *hour = bcdToDec(Wire.read() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.read());
  *dayOfMonth = bcdToDec(Wire.read());
  *month = bcdToDec(Wire.read());
  *year = bcdToDec(Wire.read());
}
//

/******************************************/
/******** Displayprompt function **********/
/*****************************************/

// This is for displaying a prompt on the LCD display

void displayprompt (String Promptmsg)
{
int xposn =0;
int charcount = Promptmsg.length();         // previous length of the String
for(int c=0; c< charcount; c=c+1)
{
lcd.setCursor(xposn, 1);
lcd.print (Promptmsg.charAt(c));
xposn ++; 
} 
delay(1000);
}


/******************************************/
/******** Displaytime function ************/
/*****************************************/
//
void displayTime()
{
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;  // retrieve data from DS3231
  
  readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,  &year);
  // send it to the serial monitor
 
  //lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(dayOfMonth, DEC);
  lcd.print("/");
  lcd.print(month, DEC);
  lcd.print("/");
//  lcd.print("20");
  lcd.print(year, DEC);
  lcd.print("  ");

  lcd.setCursor(8,0);
  lcd.print(hour, DEC);
//  if (hour <10)
//  {
//    lcd.print("0");
//    lcd.print(hour, DEC);
//  }
//  else
//  {
//   lcd.print(hour,DEC);
//   lcd.print(hour%10); 
//  }
     
  // convert the byte variable to a decimal number when displayed
  lcd.print(":");
  
  if (minute<10)
  {
    lcd.print("0");
  }
  lcd.print(minute,DEC);
//  else
//  {
//    lcd.print(minute,DEC);
//    lcd.print(minute%10);
//  }
          
 /* 
  
  lcd.print(":");
  if (second<10)
  {
    lcd.print("0"); 
    lcd.print(second, DEC);
  }
   else
   {
    lcd.print(second,DEC);
    //lcd.print(second%10);
   }
  */
  
/*
  //Serial.println (hour , DEC);                                      // For debugging 
  //Serial.println (minute , DEC);
*/
  
   
 /*
  switch(dayOfWeek){
  case 1:
    lcd.println("Sun ");
    break;
  case 2:
    lcd.println("Mon ");
    break;
  case 3:
    lcd.println("Tue ");
    break;
  case 4:
    lcd.println("Wed ");
    break;
  case 5:
    lcd.println("Thu ");
    break;
  case 6:
    lcd.println("Fri ");
    break;
  case 7:
    lcd.println("Sat ");
    break;
  }
*/
}

/***************************************************************/
/************** INTRO SCREEN DISPLAY ***************************/
/**************************************************************/
void introScreen ()
{
  lcd.setCursor(4,1);
  lcd.print("SMART SWITCH");
  lcd.setCursor(2,2);
  lcd.print("R RAMESH");
  lcd.setCursor(6,3);
  lcd.print("(C)2020");
}

 /***********************************************/
 /********* SWITCHFLICK FUNCTION ****************/
 /***********************************************/
 
//void switchFlick()
// {
// byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;                                   // retrieve data from DS3231
// readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,  &year);
// 
// if((hour >=10) && (hour <17))                                                                    // Bathroom Lights not to come on between 10 AM and 5 PM
//  {
//   digitalWrite(Relay_1, RELAY_OFF); 
//   lcd.setCursor(0,2);
//   lcd.print("BATH TUBE  DEACTIVE");    
//  }
//  else
//  {
//  digitalWrite(Relay_1, RELAY_OFF); 
//  //lcd.setCursor(0,2);
//  //lcd.print("BATH TUBE     ACTIVE");   
//  }
// }
 
 
/******************************************/
/************** LOOP function ************/
/*****************************************/

void loop() 
{
byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;                                   // retrieve data from DS3231
readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,  &year);


if((hour >=9) && (hour <= 17))
{
FlagTube = 0;                                                             // Flag to decide state of bathroom tube between 9 AM and 6 PM
FlagSolar = 1;                                                            // Flag to decide state of solar lights between 9 AM and 6 PM
}
else
{
FlagTube = 1;                                                             // Flag to decide state of bathroom tube between 9 AM and 6 PM
FlagSolar = 0;  
}

displayTime();

  // Used for calculating distance
  
  long duration, distance, dist_measure;
  digitalWrite(trigPin, LOW); 
  delayMicroseconds(2);                         //2
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(12);                        //10
  digitalWrite(trigPin, LOW);
  duration = pulseIn(echoPin, HIGH);
  dist_measure = (duration/2) / 29.1;
  
  Serial.print("Dist_Measure = ");
  Serial.println(dist_measure);
  total = total + dist_measure;
  
  readIndex = readIndex + 1;
  Serial.print ("Average Distance ");
  distance = total / readIndex;
  if (distance > 160) distance = 155;
  if (distance < 100) distance = 100;
  Serial.println (distance);
  Serial.println (" ");
  
if (readIndex >= numReadings) 
{
// ...wrap around to the beginning:
readIndex = 0;
total = 0;
}

  // displayprompt ("Distance is");
  //lcd.setCursor(15,0);
  //lcd.print("        "); 
  
  if (timer_count >0)
  {
    lcd.setCursor(14,0);
    lcd.print(" ");
    lcd.print (elapsedtimelimit-elapsedtimecount);
    lcd.print(" s");
  }
  else
  {
      if (distance<10)
      {
        lcd.setCursor(15,0);
        lcd.print("0"); 
        lcd.print(distance, DEC);
        lcd.print("cm "); 
        //Serial.print(distance);
        //Serial.println("cm");
        
      }
       else
       {
        lcd.setCursor(14,0);
        lcd.print(distance,DEC);
        lcd.print("cm ");
        //lcd.print(distance%10);
        //Serial.print(distance);
        //Serial.println("cm");
       }
  }
  delay(300);
  
  
  if (distance < 150)    
  {
    FlagPresence =1;                                                         // If a motion detected at 150cms or less from the sensor
    elapsedtimecount =0;                                                     // Reset elapsedtimecount
  }
  
  if (FlagPresence ==1)
  {
    if(FlagSolar == 1)                                                        // This is to ensure that Solar light iS only switch on in the day hours
        {
        Serial.println("Solar light ON");                                     // Line used to debug and verify what is being read
        digitalWrite(Relay_4, RELAY_ON);                                      // Set  the relay 1 on to turn on solar light
        lcd.setCursor(0,3);
        lcd.print("SOLAR LIGHT      ON");

        Serial.println("Bath Tube OFF"); 
        digitalWrite(Relay_1, RELAY_ON); 
        lcd.setCursor(0,2);
        lcd.print("BATH TUBE        OFF");

        lcd.setCursor(0,1);
        lcd.print("EXHAUST FAN      OFF");
        }

   if (FlagTube == 1)                                                          // If Tube is ON => Solar is off
        {
        Serial.println("Bath Tube ON"); 
        digitalWrite(Relay_1, RELAY_OFF); 
        lcd.setCursor(0,2);
        lcd.print("BATH TUBE       ON ");

        Serial.println("Solar light OFF");                                     // Line used to debug and verify what is being read
        digitalWrite(Relay_4, RELAY_OFF);                                      // Set  the relay 1 on to turn on solar light
        lcd.setCursor(0,3);
        lcd.print("SOLAR LIGHT     OFF");
        }
                
      timer_count=timer_count+1;
      elapsedtimecount = elapsedtimecount +1;
     
      Serial.print("TIME COUNT => ");
      Serial.println(timer_count);
      Serial.print("ELAPSED TIME COUNT => ");
      Serial.println(elapsedtimecount);
    
        if (timer_count >180)                                                    // Turn Exhaust fan on after 60 seconds
        {
        Serial.println("Exhaust Fan ON");
        digitalWrite(Relay_3, RELAY_ON); 
        lcd.setCursor(0,1);
        lcd.print("EXHAUST FAN      ON ");
        }                                         
  
  
  /**********************************************/
  /********** SWITCH OFF ROUTINE ****************/
  /**********************************************/
  
  if ((elapsedtimecount>=elapsedtimelimit)   and (distance > 150))                             // Check for 100 seconds and turn off all lights / fans if no activity detected
         {
         timer_count=0;
         elapsedtimecount =0;
         clearflag =0;
         FlagPresence =0;
         
         digitalWrite(Relay_4, RELAY_OFF); 
         lcd.setCursor(0,3);
         lcd.print("SOLAR LIGHT      OFF");
         Serial.println("Solar light OFF");
         
         digitalWrite(Relay_3, RELAY_OFF);                                       // set the Relay 3 to turn exhaust fan off
         lcd.setCursor(0,1);
         lcd.print("EXHAUST FAN      OFF");
         Serial.println("Exhaust Fan OFF");
         
         lcd.clear();
         introScreen();
         }  
  } 
 /*   
      
   while (mySerial.available()) 
  {  
    char c = mySerial.read(); //Conduct a serial read
    Serial.print (c);
    
    if (c =='#') 
    {
    break;
    }                                         //Exit the loop when the # is detected after the word
    else
    {
    voice += c;                              //Shorthand for voice = voice + c
    }
  }

 Serial.println(voice);   
 if (voice.length() > 0) 
 {
//----------Control Multiple Relays----------// 
 
  if (voice == "*lights")  
  {
   if (Flaglights==0) 
    {
  //displayprompt ("Lights coming ON  ");
  //Serial.println ("Lights ON");
  alllightson();                                    //Turn On All Relays (Call Function)
  Flaglights=1;
    }
   else 
    {
    //displayprompt ("Lights going OFF ");
    alllightsoff();                                 //Turn Off  All lights (Call Function)
    Flaglights=0;
    } 
  }
   
 //----------Turn On One-By-One----------// 
 
  else 
  if(voice == "*extfwd") 
  {
    if (Flagpin_1==0)
    {digitalWrite(Relay_1, RELAY_ON);
     Flagpin_1 =1;
    //displayprompt ("Ext Fwd Lt   ON ");
    }
    else
    {digitalWrite(Relay_1, RELAY_OFF);
    Flagpin_1 =0;
    //displayprompt ("Ext Fwd Lt   OFF");
   }
  }
  
  else 

  if(voice == "*extaft") 
  {
    if (Flagpin_2==0)
    {digitalWrite(Relay_2, RELAY_ON);
     Flagpin_2 =1;
    //displayprompt ("Ext Aft Lt ON ");
    }
    else
    {digitalWrite(Relay_2, RELAY_OFF);
    Flagpin_2 =0;
    //displayprompt ("Ext Aft Lt OFF");
   }
  }
  
  else 

  if(voice == "*sitout") 
  {
    if (Flagpin_3==0)
    {digitalWrite(Relay_3, RELAY_ON);
     Flagpin_3 =1;
    //displayprompt ("Sitout Lt ON ");
    }
    else
    {digitalWrite(Relay_3, RELAY_OFF);
    Flagpin_3 =0;
    //displayprompt ("Sitout Lt OFF");
   }
  }
  
  else 
  
  if(voice == "*tvlt") 
  {
    if (Flagpin_4==0)
    {digitalWrite(Relay_4, RELAY_ON);
     Flagpin_4 =1;
    //displayprompt ("TV Hall Lt ON     ");
    }
    else
    {digitalWrite(Relay_4, RELAY_OFF);
    Flagpin_4 =0;
    //displayprompt ("TV Hall Lt OFF    ");
   }
  }
  
  else
  if (voice == "*tvfan") 
  {
     if (Flagpin_5 ==0)
     {
    digitalWrite(Relay_5, RELAY_ON);
    Flagpin_5 =1;
    //displayprompt ("TV Hall Fan ON");
     }
     else
     {
     digitalWrite(Relay_5, RELAY_OFF);
     Flagpin_5 =0;
     //displayprompt ("TV Hall Fan OFF"); 
     }
   } 
  }                                            // end of if voice.length >0
//-----------------------------------------------------------------------//
delay(500); 
voice="";                                     //Reset the variable after initiating
*/

 }                                            // end of void loop()
      
